--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]
--Auras
local math_floor = math.floor
local _G = _G
local auraUpdateInterval = 1
local formatTime
do
    --[[
            formatTime: transform time in seconds to days/hours/minutes
            arguments:
                secs: time in seconds
            returns:
                formtated time  --]]
    function formatTime(secs)
        if secs >= 86400 then
            return format('%dd', math_floor(secs/86400 + 0.5));
        elseif secs >= 3600 then
            return format('%dh', math_floor(secs/3600 + 0.5));
        elseif secs >= 60 then
            return format('%dm', math_floor(secs/60 + 0.5));
        else
            return (math_floor(secs + 0.5));
        end
    end
end

function DHUD2:updateCombos()
    local points = GetComboPoints()
    for i = 1, 5 do
        local combo = _G["DHUD2_b"..i]
        if i <= points then
            combo:SetNormalTexture("Interface\\AddOns\\DHUD2\\textures\\c"..i)
            combo:Show()
        else
            combo:Hide()
        end
    end
end

    
function DHUD2:updatePlayerPvP()
    local p = DHUD2.db.profile
    if p.show.playerPvpCcon and DUHD2.isAlive then
        local _, _, _, _, _, _, _, x0, x1, y0, y1, path = unpack(self.layout.icons.playerPvP)
        if UnitIsPVPFreeForAll("player")  then
            path = path[3]
        elseif UnitIsPVP("player") then 
            local faction = UnitFactionGroup("player")
            if faction == "Horde" then
                path = path[2]
            else
                path = path[1]
            end
        else
            DHUD2_playerPvP:Hide()
            return
        end
        local texture = _G["DHUD2_playerPvP_Texture"]
        texture:SetTexture(path)
        texture:SetTexCoord(x0,x1,y0,y1)
        DHUD2_playerPvP:Show()
    else
        DHUD2_playerPvP:Hide()
    end
end

function DHUD2:updatePlayerAuras()
    -- Auras can be buffs or Weapon Enchants
    self:CancelAllScheduledEvents()
    local i, icon
    local j = 1
    local p = DHUD2.db.profile
    local timeToEvent = 120
    if p.show.playerBuffs then
        -- Buffs
        for i = 1, 40 do
            local buffI = GetPlayerBuff( i, "HELPFUL" )
            local timeLeft = GetPlayerBuffTimeLeft(buffI)
            if timeLeft > 0 and timeLeft <= p.gral.playerBuffTimeFilter then
                local r, g, b, hex = self:colorize("player", "aura", timeLeft / 20)
                local button = _G["DHUD2_playerBuff"..j]
                button.hasbuff = true
                button.hasweapon = false
                button.auraTimer = 0
                button.duration = math_floor(timeLeft + 0.5)
                button.id = buffI
                button:SetNormalTexture(GetPlayerBuffTexture(buffI))
                local buffBorder = _G["DHUD2_playerBuff"..j.."_Border"]
                buffBorder:SetVertexColor(r, g, b)
                buffBorder:Show()
                local buffText = _G["DHUD2_playerBuff"..j.."_Text"]
                buffText:SetText("|cff"..hex..formatTime(button.duration))
                button:SetScript("OnUpdate", self.OnUpdateAura)
                button:EnableMouse(true)
                button:Show()
                if j == 16 then
                    self:Print("You have reached the buff limit. Contact Arcanefoam and ask for more buff slots")
                    break
                else
                    j = j + 1
                end
            elseif timeLeft > 0 then
                if timeLeft - p.gral.playerBuffTimeFilter < timeToEvent then
                    timeToEvent = timeLeft - p.gral.playerBuffTimeFilter
                end
            end
            
        end
        -- Weapons
        local hasMainHandEnchant, mainHandExpiration, _, hasOffHandEnchant, offHandExpiration, _ = GetWeaponEnchantInfo()
        local id = 0
        local slot = 0
        for i = 1, 2 do
            -- Fit main and off hand
            if i == 1 and hasMainHandEnchant then
                timeLeft = mainHandExpiration/1000
                id = 1
                slot = 16
            elseif i == 2 and hasOffHandEnchant then
                timeLeft = offHandExpiration/1000
                id = 2
                slot = 17
            else
               timeLeft = 0 
            end
            if timeLeft > 0 and timeLeft <= p.gral.playerBuffTimeFilter then
                timeLeft = math_floor(timeLeft + 0.5)
                local r, g, b, hex = self:colorize("player", "aura", timeLeft / 20)
                local button = _G["DHUD2_playerBuff"..j]
                button.hasbuff = false
                button.hasweapon = true
                button.auraTimer = 0
                button.id = id
                button:SetNormalTexture(GetInventoryItemTexture("player", slot))
                local buffBorder = _G["DHUD2_playerBuff"..j.."_Border"]
                buffBorder:SetVertexColor(r, g, b)
                buffBorder:Show()
                local buffText = _G["DHUD2_playerBuff"..j.."_Text"]
                buffText:SetText("|cff"..hex..formatTime(timeLeft))
                button:SetScript("OnUpdate", self.OnUpdateAura)
                button:EnableMouse(true)
                button:Show()
                if j == 16 then
                    self:Print("You have reached the buff limit. Contact Arcanefoam and ask for more buff slots")
                    break
                else
                    j = j + 1
                end
            elseif timeLeft > 0 then
                if timeLeft - p.gral.playerBuffTimeFilter < timeToEvent then
                    timeToEvent = timeLeft - p.gral.playerBuffTimeFilter
                end
            end
        end
    end
    for i = j, 16 do
        local button = _G["DHUD2_playerBuff"..i]
        button.id = 0
        button.hasbuff = false
        button.hasweapon = false
        button:SetScript("OnUpdate", nil)
        button:EnableMouse(false)
        button:Hide()
    end
    if timeToEvent < 120 then
        -- Trigger an event for the buff with closest time to playerBuffTimeFilter
        self:ScheduleEvent("DHUD2_Auras", self.updatePlayerAuras, timeToEvent, self)
    end
end
    
function DHUD2:updateTargetAuras(enable)
    local i
    local buffBorder, buffText, buffFrame
    local debuffBorder, debuffText, debuffFrame
    local p = DHUD2.db.profile
    if not p.gral.swapTargetAuras then
        buffFrame   = "DHUD2_TRA";
        debuffFrame = "DHUD2_TLA";
    else
        debuffFrame = "DHUD2_TLA";
        buffFrame   = "DHUD2_TRA";    
    end
    local show = p.show.targetAuras and enable
    -- Buffs
    for i = 1, 40 do
        local button = _G[buffFrame..i]
        local buffName, _, buffTexture, buffApplication = UnitBuff("target", i)
        if show and buffName then
            button.hasbuff = true
            button.unit = "target"
            button.id = i
            --local icon = _G[button:GetName()]
            --icon:SetNormalTexture(buffTexture) 
            button:SetNormalTexture(buffTexture)
            buffBorder = _G[buffFrame..i.."_Border"]
            buffBorder:Hide()
            buffText = _G[buffFrame..i.."_Text"]
            if buffApplication <= 1 then
                buffText:SetText("")
            elseif buffApplication > 1 then
                buffText:SetText(buffApplication)
            end
            button:Show()
        else    
            button:Hide()
            button.hasdebuff = false
        end
    end        
    -- DeBuffs
    for i = 1, 40 do
        local button = _G[debuffFrame..i]
        local debuffName, _, debuffTexture, debuffApplication = UnitDebuff("target", i)
        if show and debuffName then
            button.hasdebuff = true
            button.unit = "target"
            button.id = i
            --local icon = _G[button:GetName()]
            --icon:SetNormalTexture(debuffTexture)
            button:SetNormalTexture(debuffTexture)
            debuffBorder = _G[debuffFrame..i.."_Border"]
            debuffBorder:Show()
            debuffText = _G[debuffFrame..i.."_Text"]
            if debuffApplication <= 1 then
                debuffText:SetText("")
            elseif debuffApplication > 1 then
                debuffText:SetText(debuffApplication)
            end
            button:Show()
        else
            button:Hide()
            button.hasdebuff = false
        end
    end
end

function DHUD2:updateTargetIcon()    
    local p = DHUD2.db.profile
    if p.show.targetPvpIcon then
        local _, _, _, _, _, _, _, x0, x1, y0, y1, path = unpack(self.layout.icons.playerPvP)
        if UnitIsPVPFreeForAll("target")  then
            path = path[3]
        elseif UnitIsPVP("target") then 
            local faction = UnitFactionGroup("target")
            if faction == "Horde" then
                path = path[2]           
            else
                path = path[1]                
            end
        else
            local index = GetRaidTargetIndex("target")
            if index then
                if index > 0 and index <= 8 then
                    path, x0, x1, y0, y1 = unpack( self.layout.raidIcons[index] )
                end
            end
        end
        local texture = _G["DHUD2_targetIcon_Texture"]
        texture:SetTexture(path)
        texture:SetTexCoord(x0,x1,y0,y1)
        DHUD2_targetIcon:Show()
    else
        DHUD2_targetIcon:Hide()
    end
end

function DHUD2:updatePetIcon(health, happiness)
    local p = DHUD2.db.profile
    --self:Debug("updatePetIcon", health, happiness)
    if health > 0 and happiness then
        local _, texture, x0, x1, y0, y1 = unpack( self.layout.images.pet[happiness] )
        if texture then
            local tex = _G["DHUD2_petHappy_Texture"]
            tex:SetTexture(texture)
            tex:SetTexCoord(x0, x1, y0, y1)
            DHUD2_petHappy:Show()
        else
            DHUD2_petHappy:Hide()
        end
    else
        DHUD2_petHappy:Hide()
    end
end
    
function DHUD2:OnUpdateAura(elapsed)
    this.auraTimer = this.auraTimer + elapsed
    if this:GetEffectiveAlpha() == 0 then
        this:EnableMouse(false)
    else            
        this:EnableMouse(true)
        if this.auraTimer >= auraUpdateInterval then
            this.auraTimer = 0
            local timeLeft = 0
            if this.hasbuff then 
                timeLeft = math_floor(GetPlayerBuffTimeLeft(this.id) + 0.5)
            elseif this.hasweapon then
                local _, mainHandExpiration, _, _, offHandExpiration, _ = GetWeaponEnchantInfo()
                if this.id == 1 and mainHandExpiration then 
                    timeLeft = mainHandExpiration/1000
                elseif this.id == 2 and offHandExpiration then
                    timeLeft = offHandExpiration/1000
                end
                if timeLeft == 0 then
                    this:SetScript("OnUpdate", nil)
                    return this:Hide()
                end
            end         
            local r, g, b, hex = DHUD2:colorize("player", "aura", timeLeft / 20)
            local buffBorder = _G[this:GetName().."_Border"]
            buffBorder:SetVertexColor(r, g, b)
            local buffText = _G[this:GetName().."_Text"]
            buffText:SetText("|cff"..hex..formatTime(timeLeft))
            if ( GameTooltip:IsOwned(this) ) then
        		GameTooltip:SetPlayerBuff(this.id)
        	end
        end
    end
end

function DHUD2:updateTargetElite()
    local p = DHUD2.db.profile
    if self:checkElite("target") and p.show.eliteIcon then
        local images = self.layout.icons
        local _, _, _, _, _, _, _, _, _, _, _, texture = unpack(images.targetElite)
        DHUD2_targetElite_Texture:SetTexture(texture)
        DHUD2_targetElite:Show()
    else
        DHUD2_targetElite:Hide()
    end
end

function DHUD2:updateTotemButton(button, startTime, duration, icon)    
    local but = _G["DHUD2_"..button]
    local butBorder = _G["DHUD2_"..button.."_Border"]
    local butText = _G["DHUD2_"..button.."_Text"]
	if ( duration > 0 ) then
        duration = math_floor(duration + 0.5)
		but:SetNormalTexture(icon)
        local r, g, b, hex = self:colorize("player", "aura", duration / 20)
        butBorder:SetVertexColor(r, g, b)
        butBorder:Show()
        butText:SetText("|cff"..hex..formatTime(duration))
        butText:Show()
        but:SetScript("OnUpdate", self.OnUpdateTotem)
        but:EnableMouse(true)
        but:Show()
        --CooldownFrame_SetTimer(buttonCooldown, startTime, duration, 1);
		--buttonCooldown:Show();
	else
		butText:Hide()
		but:SetScript("OnUpdate", nil)
		but:Hide()
	end
end

function DHUD2:OnUpdateTotem(elapsed)
    this.totemTimer = this.totemTimer + elapsed
    if this:GetEffectiveAlpha() == 0 then
        this:EnableMouse(false)
    else            
        this:EnableMouse(true)
        if this.totemTimer >= auraUpdateInterval then
            this.totemTimer = 0
            local timeLeft = math_floor(GetTotemTimeLeft(this.slot) + 0.5)
            local r, g, b, hex = DHUD2:colorize("player", "aura", timeLeft / 20)
            local buffBorder = _G[this:GetName().."_Border"]
            buffBorder:SetVertexColor(r, g, b)
            local buffText = _G[this:GetName().."_Text"]
            buffText:SetText("|cff"..hex..formatTime(timeLeft))
        	if ( GameTooltip:IsOwned(this) ) then
        		GameTooltip:SetTotem(this.slot)
        	end
        end
    end
end
