--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]

--ACE Version
local VERSION = tonumber(("$Revision 57887$"):match("%d+"))
--Addon Mix
DHUD2 = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "FuBarPlugin-2.0")
DHUD2.revision = "r" .. VERSION
DHUD2.versionstring = "0.9b |cffff8888r%d|r"
DHUD2.version = DHUD2.versionstring:format(VERSION)
DHUD2.date = ("$Date$"):match("%d%d%d%d%-%d%d%-%d%d")
local L = AceLibrary("AceLocale-2.2"):new("DHUD2")
local Tablet = AceLibrary("Tablet-2.0")
DHUD2:RegisterDB("DHUD2DB", "DHUD2DBPC")

local unpack = unpack
local pairs = pairs
local type = type
local _G = _G
local string_find = string.find
local string_format = string.format
local string_len = string.len
local string_sub = string.sub 
local string_gsub = string.gsub
local UnitHealthMax = UnitHealthMax
local UnitHealth = UnitHealth
local UnitInParty = UnitInParty
local UnitIsEnemy = UnitIsEnemy
local UnitManaMax = UnitManaMax
local UnitMana = UnitMana
local UnitIsTapped = UnitIsTapped
local UnitIsTappedByPlayer = UnitIsTappedByPlayer
local UnitPowerType = UnitPowerType
local UnitClassification = UnitClassification
local UnitIsPlayer = UnitIsPlayer
local UnitPlayerControlled = UnitPlayerControlled
local UnitCanAttack = UnitCanAttack
local UnitIsPVP = UnitIsPVP
local UnitCanAttack = UnitCanAttack
local UnitFactionGroup = UnitFactionGroup
local UnitReaction = UnitReaction
local UnitIsPVPFreeForAll = UnitIsPVPFreeForAll
local UnitDebuff = UnitDebuff
local UnitCreatureType = UnitCreatureType
local UnitIsUnit = UnitIsUnit
local UnitClass = UnitClass
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitPlayerOrPetInParty = UnitPlayerOrPetInParty 
local UnitBuff = UnitBuff
local GetPlayerBuff = GetPlayerBuff
local GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local GetRaidTargetIndex = GetRaidTargetIndex
local GetDifficultyColor = GetDifficultyColor
local GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local GetPetHappiness = GetPetHappiness

local isSetup = false
local targetBorder = ""
local petBorder = ""
local isAlive = false
local isDead = false
local playerCast = false
local inCombat = false
local isDruid = false
local isShaman = false
local hasPet = false
local regen = false
local needH
local needP
local currLayout = {
        playerHealth = "e",
        playerPower = "e",
        targetHealth = "e",
        targetPower = "e",
        petHealth = "e",
        petPower = "e",
        druidMana = "e",
        playerCastBar = "e",
        playerFlash = "e",
        targetCastBar = "e",
        targetFlash = "e",
        targetTargetHealth = "e",
        targetTargetPower = "e",
        focusHealth = "e",
        focusPower = "e",
    }

DHUD2.isDruid = isDruid
DHUD2.isShaman = isShaman
DHUD2.isAlive = isAlive
DHUD2.isDead = isDead
DHUD2.rBorder = rBorder
DHUD2.lBorder = lBorder
DHUD2.currLayout = currLayout
    
-- Local Functions
local showFrames, myAddons, formatTime, getHealth, getPower
do
    function myAddons()
        if (myAddOnsFrame_Register) then
            local DHUD2_mya = {
                ["name"]         = "DHUD2",
                ["version"]      = versionstring,
                ["author"]       = "Arcanefoam (Horacio Hoyos) -- Drathl/Silberklinge",
                ["category"]     = MYADDONS_CATEGORY_COMBAT,
                ["email"]        = "arcanefoam AT gmail DOT com",
                ["website"]      = "http://www.wowace.com",
            }
            myAddOnsFrame_Register(DHUD2_mya)
        end
    end
    
    --[[
            getHealth: Get the health value for a unit. 
            arguments:
                unit: unit to query value
            returns:
                maximum, current and percent values. nedd: bollean idicating if the unit's health is below 100% --]]
    function getHealth(unit)
        local maxval, curval, curFrac, need, found
        maxval = UnitHealthMax(unit)
        if unit == "player" or unit == "pet" or UnitIsUnit(unit, "pet") then
            curval = UnitHealth(unit)
            curFrac = curval / maxval
        else
            if UnitInParty(unit) or UnitInRaid(unit) then
                curval = UnitHealth(unit)
                curFrac = curval / maxval
            elseif UnitIsEnemy("player", unit) then
                if MobHealth3 then
                    curval, maxval, found = MobHealth3:GetUnitHealth(unit, curval, maxval)
                elseif MobHealth_GetTargetCurHP and MobHealth_GetTargetMaxHP then
                    curval = MobHealth_GetTargetCurHP()
                    maxval = MobHealth_GetTargetMaxHP()
                    found = curval and maxval
                    DHUD2:Debug("MobHealth", curval, maxval, found)
                end
                if found then
                    curFrac = curval / maxval
                else
                    curval = UnitHealth(unit)
                    curFrac = curval / 100
                end
            else
                curval = UnitHealth(unit)
                curFrac = curval / 100
            end
        end
        if curFrac ~= 1 then 
            need = true
        end
        return maxval, curval, curFrac, need
    end
    
    --[[
            getPower: Get the health value for a unit. 
            arguments:
                unit: unit to query value
                powerType: type of power for that unit
            returns:
                maximum, current and percent values. nedd: bollean idicating if the unit lackcs power (has power if power = rage) --]]
    function getPower(who)
        local maxval, curval, curFrac, need, powerType
        powerType = UnitPowerType(who)
        --self:Debug("getPower", who, powerType)
        if powerType then
            maxval = UnitManaMax(who)
            curval = UnitMana(who)
            if maxval ~= 0 then
                curFrac = curval / maxval
                if powerType == 1 then
                    if curFrac ~= 0 then 
                        need = true
                    end
                else
                    if curFrac ~= 1 then 
                        need = true
                    end
                end
            else
                curFrac = 0
                need = false
            end
            return maxval, curval, curFrac, need
        else
            return false
        end
    end

end
DHUD2.getHealth = getHealth
DHUD2.getPower = getPower

 -- General use functions
 --[[
            updateAlpha: Set the HUD's alpha acording to the state of the player
            arguments:
            returns:
                --]]
function DHUD2:updateAlpha()
    local p = self.db.profile
    local alpha
    if isDead then
        alpha = "dead"
    elseif inCombat then
        alpha = "combat"  
    elseif hasTarget then
        alpha = "select"
    elseif regen or playerCast or needH or needP then
        alpha = "regen"
    else
        alpha = "ooc"
    end
    self:addBackgroundBorder()
    DHUD2_leftFrame:SetAlpha(p.alphas[alpha])
    DHUD2_rightFrame:SetAlpha(p.alphas[alpha])
end

--[[
            rgb2hex: Transform a RGB color to a HEX color
            arguments:
                r, g, b: RGB values
            returns:
                Hex value for the RGB   --]]
function DHUD2:rgb2hex(r, g, b)
    local hexr, hexg, hexb
    hexr = string_format("%x", r * 255)
    hexr = string_len(hexr) == 1 and "0"..hexr or hexr
    hexg = string_format("%x", g * 255)
    hexg = string_len(hexg) == 1 and "0"..hexg or hexg
    hexb = string_format("%x", b * 255)
    hexb = string_len(hexb) == 1 and "0"..hexb or hexb
    return hexr..hexg..hexb
end

--[[
        colorize: Get the bar color according to the mana or power type and percent full. USed for bars and texts
        arguments:
            who: Unit's property being colorized
            propoerty: Property being colorized
            percent: Current percent value for that property
        returns:
            r, g, b and hex values for the calculated color.    --]]
function DHUD2:colorize(who, property, percent)
    local colorTable = DHUD2.db.profile.gral.colors
    if not colorTable[who] then return 0,0,0; end
    local r, g, b, diff, r0, g0, b0, r1, g1, b1, r2, g2, b2
    local threshold1, threshold2  = 0.6, 0.2
    if colorTable[who][property] then
        r0, g0, b0 = unpack(colorTable[who][property].color0)
        r1, g1, b1 = unpack(colorTable[who][property].color1)
        r2, g2, b2 = unpack(colorTable[who][property].color2)
    else
        r0, g0, b0 = unpack(colorTable.target.tapped.color0)
        r1, g1, b1 = unpack(colorTable.target.tapped.color1)
        r2, g2, b2 = unpack(colorTable.target.tapped.color2)
    end
    if ( percent <= threshold2 ) then
        r, g, b = r2, g2, b2
    elseif ( percent <= threshold1) then
        diff = 1 - (percent - threshold2) / (threshold1 - threshold2);
        r = r1 - (r1 - r2) * diff;        
        g = g1 - (g1 - g2) * diff;        
        b = b1 - (b1 - b2) * diff;    
    elseif ( percent < 1) then        
        diff = 1 - (percent - threshold1) / (1 - threshold1);        
        r = r0 - (r0 - r1) * diff;        
        g = g0 - (g0 - g1) * diff;        
        b = b0 - (b0 - b1) * diff;    
    else
        r, g, b = r0, g0, b0
    end
    self:Debug("Final:", r, g, b)
    if (r < 0) then r = 0; end    
    if (r > 1) then r = 1; end    
    if (g < 0) then g = 0; end    
    if (g > 1) then g = 1; end    
    if (b < 0) then b = 0; end    
    if (b > 1) then b = 1; end
    hex = self:rgb2hex(r, g, b)
    return r, g, b, hex
end

--[[
        checkElite: Check if the unit is elite
        arguments:
            unit: unit to check
        returns:
            icon to use, text to show --]]
function DHUD2:checkElite(unit)
    local el = UnitClassification(unit)
    if el then
        local eliteValues = {
            ["worldboss"] = {"targetElite", "++"},
            ["elite"] = {"targetElite", "+"},
            ["rareelite"] = {"targetRare", "r+"},
            ["rare"] = {"targetRare", "r"},
            ["normal"] = {false, ""},
            }
        return eliteValues[el][1], eliteValues[el][2]
    else
        return eliteValues["normal"][1], eliteValues["normal"][2]
    end
end

 --[[
        unitIsNPC: Check if the unit is a NPC
        arguments:
            unit: unit to check
        returns:
            true if the unit is a NPC --]]
function DHUD2:unitIsNPC(unit)
    return not (UnitIsPlayer(unit) or UnitPlayerControlled(unit) or UnitCanAttack("player", unit))
end
--[[
        unitIsPet: Check if the unit is a pet
        arguments:
            unit: unit to check
        returns:
            true if the unit is a pet --]]
function DHUD2:unitIsPet(unit)
    return not self:unitIsNPC(unit) and not UnitIsPlayer(unit) and UnitPlayerControlled(unit)
end   

function DHUD2:OnInitialize()
    self:createFrames()
    --FuBar
    self.hasIcon = true
    self:SetIcon("Interface\\AddOns\\DHUD2\\icon.tga")
    self.OnMenuRequest = self.options

    -- MyAddons Support
    myAddons()
end

function DHUD2:OnClick()
    if AceLibrary:HasInstance("Waterfall-1.0") then
        AceLibrary("Waterfall-1.0"):Open('DHUD2')
	end
end

function DHUD2:blizzFrames(showPlayer, showTarget, showPet, showCasting)
    if showTarget then
        TargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
        TargetFrame:RegisterEvent("UNIT_HEALTH")
        TargetFrame:RegisterEvent("UNIT_LEVEL")
        TargetFrame:RegisterEvent("UNIT_FACTION")
        TargetFrame:RegisterEvent("UNIT_CLASSIFICATION_CHANGED")
        TargetFrame:RegisterEvent("UNIT_AURA")
        TargetFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
        TargetFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
        TargetFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
        if UnitExists("target") then TargetFrame:Show() end
        ComboFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
        ComboFrame:RegisterEvent("PLAYER_COMBO_POINTS")
    else
        TargetFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
        TargetFrame:UnregisterEvent("UNIT_HEALTH")
        TargetFrame:UnregisterEvent("UNIT_LEVEL")
        TargetFrame:UnregisterEvent("UNIT_FACTION")
        TargetFrame:UnregisterEvent("UNIT_CLASSIFICATION_CHANGED")
        TargetFrame:UnregisterEvent("UNIT_AURA")
        TargetFrame:UnregisterEvent("PLAYER_FLAGS_CHANGED")
        TargetFrame:UnregisterEvent("PARTY_MEMBERS_CHANGED")
        TargetFrame:UnregisterEvent("PLAYER_FOCUS_CHANGED")
        ComboFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
        ComboFrame:UnregisterEvent("PLAYER_COMBO_POINTS")
        TargetFrame:Hide()
        ComboFrame:Hide()
    end
    
    if showPlayer then
        PlayerFrame:RegisterEvent("UNIT_LEVEL")
        PlayerFrame:RegisterEvent("UNIT_COMBAT")
        PlayerFrame:RegisterEvent("UNIT_SPELLMISS")
        PlayerFrame:RegisterEvent("UNIT_PVP_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_MAXMANA")
        PlayerFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
        PlayerFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
        PlayerFrame:RegisterEvent("PLAYER_UPDATE_RESTING")
        PlayerFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
        PlayerFrame:RegisterEvent("PARTY_LEADER_CHANGED")
        PlayerFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
        PlayerFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
        PlayerFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
        PlayerFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
        PlayerFrameHealthBar:RegisterEvent("UNIT_HEALTH")
        PlayerFrameHealthBar:RegisterEvent("UNIT_MAXHEALTH")
        PlayerFrameManaBar:RegisterEvent("UNIT_MANA")
        PlayerFrameManaBar:RegisterEvent("UNIT_RAGE")
        PlayerFrameManaBar:RegisterEvent("UNIT_FOCUS")
        PlayerFrameManaBar:RegisterEvent("UNIT_ENERGY")
        PlayerFrameManaBar:RegisterEvent("UNIT_HAPPINESS")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXMANA")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXRAGE")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXFOCUS")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXENERGY")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXHAPPINESS")
        PlayerFrameManaBar:RegisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:RegisterEvent("UNIT_NAME_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:Show()
    else
        PlayerFrame:UnregisterEvent("UNIT_LEVEL")
        PlayerFrame:UnregisterEvent("UNIT_COMBAT")
        PlayerFrame:UnregisterEvent("UNIT_SPELLMISS")
        PlayerFrame:UnregisterEvent("UNIT_PVP_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_MAXMANA")
        PlayerFrame:UnregisterEvent("PLAYER_ENTER_COMBAT")
        PlayerFrame:UnregisterEvent("PLAYER_LEAVE_COMBAT")
        PlayerFrame:UnregisterEvent("PLAYER_UPDATE_RESTING")
        PlayerFrame:UnregisterEvent("PARTY_MEMBERS_CHANGED")
        PlayerFrame:UnregisterEvent("PARTY_LEADER_CHANGED")
        PlayerFrame:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED")
        PlayerFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
        PlayerFrame:UnregisterEvent("PLAYER_REGEN_DISABLED")
        PlayerFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")
        PlayerFrameHealthBar:UnregisterEvent("UNIT_HEALTH")
        PlayerFrameHealthBar:UnregisterEvent("UNIT_MAXHEALTH")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MANA")
        PlayerFrameManaBar:UnregisterEvent("UNIT_RAGE")
        PlayerFrameManaBar:UnregisterEvent("UNIT_FOCUS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_ENERGY")
        PlayerFrameManaBar:UnregisterEvent("UNIT_HAPPINESS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXMANA")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXRAGE")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXFOCUS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXENERGY")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXHAPPINESS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:UnregisterEvent("UNIT_NAME_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_PORTRAIT_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:Hide()
    end    
    
    if showCasting then
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
    else
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")        
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
        CastingBarFrame:Hide()
    end
    
    if showPet then
        
    end
    
end

-- Asing each bar the their respective tracking stat accordingly to the saved profile
function asignBars()
    local l = DHUD2.db.profile.layout
    -- Player Bars
    if currLayout.playerHealth ~= "e" then
        _G["DHUD2_"..currLayout.playerHealth]:EnableMouse(false)
        _G["DHUD2_"..currLayout.playerHealth]:RegisterForClicks()
    end
    if l.playerBars == "HL~PR" then
        currLayout.playerHealth = "li"
        currLayout.playerPower = "ri"
        currLayout.targetHealth = "lm"
        currLayout.targetPower = "rm"
        currLayout.playerCastBar = "rc"
        currLayout.playerFlash = "rf"
        currLayout.targetCastBar = "lc"
        currLayout.targetFlash = "lf"        
    elseif l.playerBars == "HR~PL" then
        currLayout.playerHealth = "ri"
        currLayout.playerPower = "li"
        currLayout.targetHealth = "rm"
        currLayout.targetPower = "lm"
        currLayout.playerCastBar = "lc"
        currLayout.playerFlash = "lf"
        currLayout.targetCastBar = "rc"
        currLayout.targetFlash = "rf"  
    elseif l.playerBars == "HI~PM" then
        if l.player == L["Left"] then
            currLayout.playerHealth = "li"
            currLayout.playerPower = "lm"
            currLayout.playerCastBar = "lc"
            currLayout.playerFlash = "lf"
            currLayout.targetHealth = "ri"
            currLayout.targetPower = "rm"
            currLayout.targetCastBar = "rc"
            currLayout.targetFlash = "rf"
        else
            currLayout.playerHealth = "ri"
            currLayout.playerPower = "rm"
            currLayout.playerCastBar = "rc"
            currLayout.playerFlash = "rf"
            currLayout.targetHealth = "li"
            currLayout.targetPower = "lm"
            currLayout.targetCastBar = "lc"
            currLayout.targetFlash = "lf"
        end
    elseif l.playerBars == "HM~PI" then
        if l.player == L["Left"] then
            currLayout.playerHealth = "lm"
            currLayout.playerPower = "li"
            currLayout.playerCastBar = "lc"
            currLayout.playerFlash = "lf"
            currLayout.targetHealth = "rm"
            currLayout.targetPower = "ri"
            currLayout.targetCastBar = "rc"
            currLayout.targetFlash = "rf"
        else
            currLayout.playerHealth = "rm"
            currLayout.playerPower = "ri"
            currLayout.playerCastBar = "rc"
            currLayout.playerFlash = "rf"
            currLayout.targetHealth = "lm"
            currLayout.targetPower = "li"
            currLayout.targetCastBar = "lc"
            currLayout.targetFlash = "lf"
        end
    end
    -- Once the player bar is defined, register it for clicks
    --[[
    _G["DHUD2_"..currLayout.playerHealth]:RegisterForClicks('RightButtonUp')
    _G["DHUD2_"..currLayout.playerHealth]:SetScript("OnClick", function(this, mouseButton)
        if (mouseButton == "RightButton") then
            ToggleDropDownMenu(1, nil, DHUD2_playerDropDown, this, 25, 10)
        end
    end )
    _G["DHUD2_"..currLayout.playerHealth]:EnableMouse(true)
    --]]
    
    
    
    -- Pet bars
    --self:Debug("petBars: ",l.petBars )
    if l.petBars == "HL~PR" then
        currLayout.petHealth = "lpo"
        currLayout.petPower = "rpo"
    elseif l.petBars == "HI~PO" then
        if l.pet == L["Left"] then
            currLayout.petHealth = "lpi"
            currLayout.petPower = "lpo"
        else
            currLayout.petHealth = "rpi"
            currLayout.petPower = "rpo"
        end
    elseif l.petBars == "HO~PI" then
        if l.pet == L["Left"] then
            currLayout.petHealth = "lpo"
            currLayout.petPower = "lpi"
        else
            currLayout.petHealth = "rpo"
            currLayout.petPower = "rpi"
        end
    end
    
    -- Druid bar
    if l.druidBar == "R" then
        currLayout.druidMana = "rpo"
    else
        currLayout.druidMana = "lpo"
    end
    
    -- Outer bars
    local outer = DHUD2.db.profile.outer
    if outer.right == "TTH" then
        currLayout.targetTargetHealth = "ro"
    elseif outer.left == "TTH" then
        currLayout.targetTargetHealth = "lo"
    else
        currLayout.targetTargetHealth = "e"
    end
    if outer.right == "TTP" then
        currLayout.targetTargetPower = "ro"
    elseif outer.left == "TTP" then
        currLayout.targetTargetPower = "lo"
    else
        currLayout.targetTargetPower = "e"
    end    
    if outer.left == "FH" then
        currLayout.focusHealth = "lo"
    elseif outer.right == "FH" then
        currLayout.focusHealth = "ro"
    else
        currLayout.focusHealth = "e"
    end
    if outer.left == "FP" then
        currLayout.focusPower = "lo"
    elseif outer.right == "FP" then
        currLayout.focusPower = "ro"
    else
        currLayout.focusPower = "e"
    end
end

FIRE_TOTEM_SLOT = 1;
EARTH_TOTEM_SLOT = 2;
WATER_TOTEM_SLOT = 3;
AIR_TOTEM_SLOT = 4;

MAX_TOTEMS = 4;

TOTEM_PRIORITIES =
{
	AIR_TOTEM_SLOT,
	WATER_TOTEM_SLOT,
	EARTH_TOTEM_SLOT,
	FIRE_TOTEM_SLOT
};
-- Thanks to bliss totem frames
function DHUD2:totemUpdate()
	local haveTotem, name, startTime, duration, icon
	local slot
	local button
	local buttonIndex = 1
	for i=1, MAX_TOTEMS do
		slot = TOTEM_PRIORITIES[i];
		haveTotem, name, startTime, duration, icon = GetTotemInfo(slot)
		if haveTotem then
			button = _G["DHUD2_b"..buttonIndex]
			button.slot = slot
            button.totemTimer = 0
			self:updateTotemButton("b"..buttonIndex, startTime, duration, icon)
			buttonIndex = buttonIndex + 1
		else
			button = _G["DHUD2_b"..MAX_TOTEMS - i + buttonIndex]
			button.slot = 0
			button:Hide()
		end
        self:Debug("totemUpdate", buttonIndex-1, button:IsVisible())
	end
end

-- Initializa the respective bars with their values and hide the unused ones
function DHUD2:setup()
    self:Debug("setup")
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile
    
    -- Set the HUD scale
    DHUD2_mainFrame:SetScale(p.gral.scale)
    
    -- Designate de bars accordingly to the profile
    asignBars()
    self:initBorders()
    local _, class = UnitClass("player")
    self:updateBar("lpi", false)
    self:updateBar("lpo", false)
    self:updateBar("rpi", false)
    self:updateBar("rpo", false)
    if class == "DRUID" then
        -- Init druid bar
        isDruid = true
        if p.show.druidBar and SmartyCat then 
            self:RegisterEvent("UNIT_DISPLAYPOWER")
            if UnitPowerType("player") ~= 0 then                
                _G["DHUD2_"..currLayout.druidMana]:Show()
                self:RegisterEvent("SmartyCat_ManaUpdated", "OnFormUpdateMana")
                curVal, maxVal = SmartyCat.GetCurrMana(), SmartyCat.GetMaxMana()
                color.r, color.g, color.b, color.hex = self:colorize("player", 0,  curVal/maxVal)
                self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color)
            end
        end
    else
        --Init Pet frames
        hasPet = UnitExists("pet")
        self:Debug("Init pet bars")
        if hasPet and p.show.pet then
            happ = GetPetHappiness()
            maxVal, curVal, curFrac, need = getHealth("pet")
            color.r, color.g, color.b, color.hex = self:colorize("pet", "health",  curFrac)
            self:updateBar(currLayout.petHealth, maxVal, curVal, curFrac, color)
            self:updatePetIcon(petCurH, happ)
            maxVal, curVal, curFrac, need = getPower("pet")
            color.r, color.g, color.b, color.hex = self:colorize("pet", UnitPowerType("pet"),  curFrac)
            self:updateBar(currLayout.petPower, maxVal, curVal, curFrac, color)
        else
            self:updatePetIcon(0, false)
            self:updateBar(currLayout.petHealth, false)
            self:updateBar(currLayout.petPower, false)
        end
    end
    
    -- Init Player Bars
    self:Debug("Init player bars")
    isAlive = not UnitIsDeadOrGhost("player")
    isDead = not isAlive
    maxVal, curVal, curFrac, need = getHealth("player")
    color.r, color.g, color.b, color.hex = self:colorize("player", "health",  curFrac)
    self:updateBar(currLayout.playerHealth, maxVal, curVal, curFrac, color)
    maxVal, curVal, curFrac, playerNeedP = getPower("player")
    color.r, color.g, color.b, color.hex = self:colorize("player", UnitPowerType("player"),  curFrac)
    self:updateBar(currLayout.playerPower, maxVal, curVal, curFrac, color)
    if class == "SHAMAN" then
        isShaman = true
        self:RegisterEvent("PLAYER_TOTEM_UPDATE")
        self:totemUpdate()
    else
        self:updateCombos()
    end
    self:updatePlayerPvP()
    if IsResting() and p.show.restIcon then
        DHUD2_playerResting:Show()
    else
        DHUD2_playerResting:Hide()
    end
    if UnitPlayerOrPetInParty("player") then
        if IsPartyLeader() then
            DHUD2_playerLeader:Show()
            local lootmethod = GetLootMethod()
            if lootmethod == "master" then
                DHUD2_playerLooter:Show()
            else
                DHUD2_playerLooter:Hide()
            end
        else
            DHUD2_playerLeader:Hide()
            DHUD2_playerLooter:Hide()
        end
    else
        DHUD2_playerLeader:Hide()
        DHUD2_playerLooter:Hide()
    end        
    DHUD2_playerInCombat:Hide()
    
    -- Init Target frames
    self:Debug("Init target bars")
    if UnitExists("target") and p.show.target then
        maxVal, curVal, curFrac = getHealth("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", "health",  curFrac)
        self:updateBar(currLayout.targetHealth, maxVal, curVal, curFrac, color)
        maxVal, curVal, curFrac = getPower("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", UnitPowerType("target"),  curFrac)
        self:updateBar(currLayout.targetPower, maxVal, curVal, curFrac, color)
        self:updateTargetElite()
        self:updateText("target", p.show.target)
        self:updateTargetAuras(true)
    else
        self:updateBar(currLayout.targetHealth, false)      
        self:updateBar(currLayout.targetPower, false)      
        self:updateTargetAuras(false)
        DHUD2_targetElite:Hide()
        self:updateText("target", false)
    end
    -- Init Target Target
    self:updateText("targetTarget", UnitExists("targetTarget") and p.show.targetTargetName)
    
    -- Outer Bars
    self:updateBar("lo", false)
    self:updateBar("ro", false)
    if UnitExists("targettarget") then
        if currLayout.targetTargetHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", "health",  curFrac)
            self:updateBar(currLayout.targetTargetHealth, maxVal, curVal, curFrac, color)            
        end
        if currLayout.targetTargetPower ~= "e" then
            maxVal, curVal, curFrac = getPower("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", UnitPowerType("targetarget"),  curFrac)
            self:updateBar(currLayout.targetTargetPower, maxVal, curVal, curFrac, color)            
        end
    end
    if UnitExists("focus") then
        if currLayout.focusHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", "health",  curFrac)
            self:updateBar(currLayout.focusHealth, maxVal, curVal, curFrac, color)            
        end
        if currLayout.focusPower ~= "e" then
            maxVal, curVal, curFrac = getPower("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", UnitPowerType("focus"), curFrac)
            self:updateBar(currLayout.focusPower, maxVal, curVal, curFrac, color)            
        end
    end
 
    -- Cast Bars
    self:Debug("Init Cast Bars")
    DHUD2_lc:Hide()
    DHUD2_rc:Hide()
    DHUD2_lcf:Hide()
    DHUD2_rcf:Hide()
    if p.show.casting then
        self:RegisterEvent("UNIT_SPELLCAST_START")
        self:RegisterEvent("UNIT_SPELLCAST_STOP")
        self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
        self:RegisterEvent("UNIT_SPELLCAST_FAILED", "OnSpellFail")
        self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "OnSpellFail")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
    else
        self:UnregisterEvent("UNIT_SPELLCAST_START")
        self:UnregisterEvent("UNIT_SPELLCAST_STOP")
        self:UnregisterEvent("UNIT_SPELLCAST_DELAYED")
        self:UnregisterEvent("UNIT_SPELLCAST_FAILED", "OnSpellFail")
        self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED", "OnSpellFail")
        self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
        self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
        self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
    end
    
    --Set fonts
    self:setFonts()
    
    -- Manage Blizz Frames 
    self:blizzFrames(p.show.blizPlayer, p.show.blizTarget, p.show.blizPet, p.show.blizCasting)
    
    self:updateAlpha()
    isSetup = true
end

function targetDropDown_Init()
    local menu;
	local name;
	local id = nil;
	if ( UnitIsUnit("target", "player") ) then
		menu = "SELF";
	elseif ( UnitIsUnit("target", "pet") ) then
		menu = "PET";
	elseif ( UnitIsPlayer("target") ) then
		id = UnitInRaid("target")
		if ( id ) then
			menu = "RAID_PLAYER"
			name = GetRaidRosterInfo(id +1);
		elseif ( UnitInParty("target") ) then
			menu = "PARTY"
		else
			menu = "PLAYER"
		end
	else
		menu = "RAID_TARGET_ICON"
		name = RAID_TARGET_ICON
	end
	if ( menu ) then
		UnitPopup_ShowMenu(DHUD2_targetDropDown, menu, "target", name, id)
	end
end


function playerDropDown_Init()
    UnitPopup_ShowMenu( DHUD2_playerDropDown, "SELF", "player" )
end
    
function DHUD2:OnEnable() -- ok
    --self:SetDebugging(true)
    self:Debug("OnEnable")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_AURAS_CHANGED")
    self:RegisterEvent("PLAYER_COMBO_POINTS")
    self:RegisterEvent("PLAYER_ALIVE")
    self:RegisterEvent("PLAYER_UNGHOST")
    self:RegisterEvent("PLAYER_DEAD")
    self:RegisterEvent("PLAYER_UPDATE_RESTING")
    self:RegisterEvent("PLAYER_FOCUS_CHANGED")
    self:RegisterEvent("UNIT_HEALTH", "onUnitHealth")
    self:RegisterEvent("UNIT_MAXHEALTH", "onUnitHealth")
    self:RegisterEvent("UNIT_MANA", "onUnitPower")
    self:RegisterEvent("UNIT_MAXMANA", "onUnitPower")  
    self:RegisterEvent("UNIT_FOCUS", "onUnitPower")
    self:RegisterEvent("UNIT_MAXFOCUS", "onUnitPower")
    self:RegisterEvent("UNIT_RAGE", "onUnitPower")
    self:RegisterEvent("UNIT_MAXRAGE", "onUnitPower")
    self:RegisterEvent("UNIT_ENERGY", "onUnitPower")
    self:RegisterEvent("UNIT_MAXENERGY", "onUnitPower")
    self:RegisterEvent("UNIT_FACTION")
    self:RegisterEvent("UNIT_AURA")
    self:RegisterEvent("UNIT_TARGET")
    self:RegisterEvent("UNIT_HAPPINESS")
    self:RegisterEvent("UNIT_PET")
    self:RegisterEvent("UNIT_DISPLAYPOWER")
    self:RegisterEvent("RAID_TARGET_UPDATE")
    self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self:RegisterEvent("UNIT_DAMAGE")
    if AceLibrary:HasInstance("Waterfall-1.0") then
        AceLibrary("Waterfall-1.0"):Register('DHUD2',
            'aceOptions', DHUD2.options,
            'title', "DHUD2",
            'colorR', 0.2, 'colorG', 0.8, 'colorB', 0.2
        )
        self:RegisterChatCommand({"/dhud2"}, function()
            AceLibrary("Waterfall-1.0"):Open('DHUD2')
        end)
        self:RegisterChatCommand({"/dhud2cl"}, self.options)
    else
        self:Debug("OnInitialize, None")
        self:RegisterChatCommand({"/dhud2"}, self.options)
    end
    
    -- Player/Target Dropdown
    UIDropDownMenu_Initialize(DHUD2_playerDropDown, playerDropDown_Init, "MENU")
    UIDropDownMenu_Initialize(DHUD2_targetDropDown, targetDropDown_Init, "MENU")
    DHUD2_target_Text:RegisterForClicks('RightButtonUp')
    DHUD2_target_Text:SetScript("OnClick", function(this, mouseButton)
        if (mouseButton == "RightButton") then
            ToggleDropDownMenu(1, nil, DHUD2_targetDropDown, this, 25, 10)
        end
    end )
    
    -- Combos/Totems
    for i = 1, 5 do
        _G["DHUD2_b"..i]:RegisterForClicks('LeftButtonUp')
    end
    
end

--Reset the init variables so the addon will reload correctly when re-enabled
function DHUD2:OnDisable()
    self:Debug("OnDisable")
    isSetup = false
    isInit = false
    self:UnregisterAllEvents()
end


function DHUD2:UNIT_DAMAGE(who)
    self:Debug("UNIT_DAMAGE", who)
    if who == "player" then
        self:updatePlayerAuras()
        self:updateAlpha()
    end
end
----
    
-- Load de configuration menu/window libraries and do initial setup
function DHUD2:PLAYER_ENTERING_WORLD()
    self:Debug("PLAYER_ENTERING_WORLD")
    if isSetup then
        return
    end  
    self:setup()
end

-- Update Target and TargetTarget
function DHUD2:PLAYER_TARGET_CHANGED()
    self:Debug("PLAYER_TARGET_CHANGED")
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile
    self:updateTargetAuras(true)
    if UnitExists("target") and p.show.target and not(self:unitIsNPC("target") and not p.show.npc) then
        hasTarget = true
        maxVal, curVal, curFrac = getHealth("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", "health", curVal)
        self:updateBar(currLayout.targetHealth, maxVal, curVal, curFrac, color)
        maxVal, curVal, curFrac, _ = getPower("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", UnitPowerType("target"), curVal)
        self:updateBar(currLayout.targetPower, maxVal, curVal, curFrac, color)
        -- Casting
        local spell, _, _, _, startTime, endTime = UnitCastingInfo("target")
        if spell then
            self:initCast(currLayout.targetCastBar, startTime, endTime, true)
        else
            spell, _, _, _, startTime, endTime = UnitChannelInfo("target")
            if spell then
                self:initCast(currLayout.targetCastBar, startTime, endTime)
            else
                _G["DHUD2_"..currLayout.targetCastBar]:Hide()
                _G["DHUD2_"..currLayout.targetCastBar.."f"]:Hide()
            end
        end
        if not isShaman then
            self:updateCombos()
        end
        self:updateText("target", p.show.target)
        self:updateTargetElite()
        -- make name clickable?
        --[[if (not self:unitIsNPC("target")) ( UnitIsUnit("target", "player") and ( GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 )) or
           ( UnitIsPlayer("target")  and not UnitIsEnemy("player", "target")  and not UnitIsUnit("target", "player") ) or 
           UnitIsUnit("target", "pet") then --]]
            DHUD2_target_Text:EnableMouse(true)
        --[[else
            DHUD2_target_Text:EnableMouse(false)
        end--]]
    else
        hasTarget = false
        if not isShaman then
            self:updateCombos()
        end
        self:updateBar(currLayout.targetHealth, false)
        self:updateBar(currLayout.targetPower, false)
        _G["DHUD2_"..currLayout.targetCastBar]:Hide()
        _G["DHUD2_"..currLayout.targetCastBar.."f"]:Hide()
        _G["DHUD2_"..currLayout.targetCastBar]:Hide()
        _G["DHUD2_"..currLayout.targetCastBar.."_Text"]:Hide()
        _G["DHUD2_"..currLayout.targetCastBar.."d_Text"]:Hide()
        DHUD2_targetElite:Hide()
        DHUD2_target_Text:Hide()
        DHUD2_target_Text:EnableMouse(false)
        DHUD2_targetTarget_Text:Hide()
    end
    self:updateTargetIcon(hasTarget)
    self:updateTargetAuras(hasTarget)
    --Target target
    if UnitExists("targettarget") and not(self:unitIsNPC("targettarget") and not p.show.npc) and not UnitIsUnit("targettarget", "player") then
        if currLayout.targetTargetHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", "health", curVal)
            self:updateBar(currLayout.targetTargetHealth, maxVal, curVal, curFrac, color)
        end
        if currLayout.targetTargetPower ~= "e" then
            maxVal, curVal, curFrac, _ = getPower("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", UnitPowerType("targettarget"), curVal)
            self:updateBar(currLayout.targetTargetPower, maxVal, curVal, curFrac, color)
        end
        self:updateText("targetTarget", p.show.targetTargetName)
    else
        if currLayout.targetTargetHealth ~= "e" then
            self:updateBar(currLayout.targetTargetHealth, false)
        end
        if currLayout.targetTargetPower ~= "e" then
            self:updateBar(currLayout.targetTargetPower, false)
        end
        self:updateText("targetTarget", false)
    end
    self:updateAlpha()
end

function DHUD2:PLAYER_REGEN_ENABLED()
    self:Debug("PLAYER_REGEN_ENABLED")
    regen = true
    inCombat = false
    DHUD2_playerInCombat:Hide()
    self:updateAlpha()
end

function DHUD2:PLAYER_REGEN_DISABLED()
    self:Debug("PLAYER_REGEN_DISABLED")
    regen = false
    inCombat = true
    if DHUD2.db.profile.show.combatIcon then
        DHUD2_playerInCombat:Show()
    end
    self:updatePlayerAuras()
    self:updateAlpha()
end  

function DHUD2:PLAYER_AURAS_CHANGED()
    self:Debug("PLAYER_AURAS_CHANGED")
    self:updatePlayerAuras()
    self:updateAlpha()
end

function DHUD2:PLAYER_COMBO_POINTS()
    self:Debug("PLAYER_COMBO_POINTS")
    self:updateCombos()
end

function DHUD2:PLAYER_ALIVE()
    self:Debug("PLAYER_ALIVE")
    if UnitIsDeadOrGhost("player") then
        self:PLAYER_DEAD()
    else
        self:OnPlayerLife()
    end
end

function DHUD2:OnPlayerLife()
    self:Debug("OnPlayerLife")
    isAlive = true
    isDead = false
    self:updateAlpha()
end

function DHUD2:PLAYER_UNGHOST()
    self:Debug("PLAYER_UNGHOST")
    isAlive = true
    isDead = false
    self:updateAlpha()
end

function DHUD2:PLAYER_DEAD()
    self:Debug("PLAYER_DEAD")
    -- First event when killed
    inCombat = false
    isDead = true
    isAlive = false
    needH = false
    needP = false
    isCasting = false
    self:updateAlpha()
end  

function DHUD2:PLAYER_UPDATE_RESTING()
    self:Debug("PLAYER_UPDATE_RESTING")
    if IsResting() and DHUD2.db.profile.show.restIcon and isAlive then
        DHUD2_playerResting:Show();
    else
        DHUD2_playerResting:Hide();
    end
end

function DHUD2:PLAYER_FOCUS_CHANGED()
    self:Debug("PLAYER_FOCUS_CHANGED")
    if UnitExists("focus") then
        local color = {r=0,g=0,b=0,hex=""}
        local maxVal, curVal, curFrac
        if currLayout.focusHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", "health",  curFrac)
            self:updateBar(currLayout.focusHealth, maxVal, curVal, curFrac, color) 
        end
        if currLayout.focusPower ~= "e" then
            maxVal, curVal, curFrac = getPower("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", UnitPowerType("focus"), curFrac)
            self:updateBar(currLayout.focusPower, maxVal, curVal, curFrac, color)
        end
    else
        if currLayout.focusHealth ~= "e" then
            self:updateBar(currLayout.focusHealth, false)            
        end
        if currLayout.focusPower ~= "e" then
            self:updateBar(currLayout.focusPower, false)            
        end
    end
    self:updateAlpha()
end

function DHUD2:PLAYER_TOTEM_UPDATE()
    self:Debug("PLAYER_TOTEM_UPDATE")
    DHUD2:totemUpdate()
end

function DHUD2:onUnitHealth(who)
    --DHUD2:Debug("onUnitHealth", who)
    local maxVal, curVal, curFrac
    local p = DHUD2.db.profile
    local color = {r=0,g=0,b=0,hex=""}
    local typ = "health"
    if who == "player" then
        maxVal, curVal, curFrac, needH = getHealth("player")
        color.r, color.g, color.b, color.hex = self:colorize("player", typ, curVal)
        self:updateBar(currLayout.playerHealth, maxVal, curVal, curFrac, color)
    elseif who == "target" and p.show.target and not (not p.show.npc and unitIsNPC("target")) then
        maxVal, curVal, curFrac = getHealth("target")
        if (UnitIsTapped("target") and (not UnitIsTappedByPlayer("target"))) then               
              typ = "tapped"
        end
        color.r, color.g, color.b, color.hex = self:colorize("target", typ, curVal)
        self:updateBar(currLayout.targetHealth, maxVal, curVal, curFrac, color)
    elseif who == "pet" and hasPet and p.show.pet then
        maxVal, curVal, curFrac, needH = getHealth("pet")
        color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curVal)
        self:updateBar(currLayout.petHealth, maxVal, curVal, curFrac, color)
    end
    if UnitExists("targettarget") and not UnitIsUnit("targettarget", "player") then
        if currLayout.targetTargetHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("targettarget")
            if (UnitIsTapped("targettarget") and (not UnitIsTappedByPlayer("targettarget"))) then               
                  typ = "tapped"
            end
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", typ,  curFrac)
            self:updateBar(currLayout.targetTargetHealth, maxVal, curVal, curFrac, color)            
        end
    end
    if UnitExists("focus") then
        if currLayout.focusHealth ~= "e" then
            maxVal, curVal, curFrac = getHealth("focus")
            if (UnitIsTapped("focus") and (not UnitIsTappedByPlayer("focus"))) then               
                  typ = "tapped"
            end
            color.r, color.g, color.b, color.hex = self:colorize("focus", typ,  curFrac)
            self:updateBar(currLayout.focusHealth, maxVal, curVal, curFrac, color)            
        end

    end
    DHUD2:updateAlpha()
end

function DHUD2:onUnitPower(who)
    --DHUD2:Debug("onUnitPower", who)
    local maxVal, curVal, curFrac
    local p = DHUD2.db.profile
    local color = {r=0,g=0,b=0,hex=""}
    local typ = UnitPowerType(who)
    if who == "player" then
        maxVal, curVal, curFrac, needP = getPower("player")
        color.r, color.g, color.b, color.hex = self:colorize("player", typ, curFrac)
        self:updateBar(currLayout.playerPower, maxVal, curVal, curFrac, color)
    elseif who == "target" and p.show.target then
        maxVal, curVal, curFrac = getPower("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", typ, curFrac)
        self:updateBar(currLayout.targetPower, maxVal, curVal, curFrac, color)
    elseif who == "pet" and hasPet and p.show.pet then
        maxVal, curVal, curFrac, needP = getPower("pet")
        color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curFrac)
        self:updateBar(currLayout.petPower, maxVal, curVal, curFrac, color)
    end
    if UnitExists("targettarget") and not UnitIsUnit("targettarget", "player") then  
        if currLayout.targetTargetPower ~= "e" then
            maxVal, curVal, curFrac = getPower("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", typ, curFrac)
            self:updateBar(currLayout.targetTargetPower, maxVal, curVal, curFrac, color)            
        end
    end
    if UnitExists("focus") then
        if currLayout.focusPower ~= "e" then
            maxVal, curVal, curFrac = getPower("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", typ, curFrac)
            self:updateBar(currLayout.focusPower, maxVal, curVal, curFrac, color)            
        end
    end
    DHUD2:updateAlpha()
end

function DHUD2:UNIT_FACTION(who)
    local p = DHUD2.db.profile
    if who == "player" then
        self:updatePlayerPvP()
        if UnitIsUnit("player", "target") then
            self:updateTargetIcon()
        end
    elseif who == "target" and p.show.target then
        self:updateTargetIcon()
    end
end

function DHUD2:UNIT_AURA(who)
    if who == "target" and DHUD2.db.profile.show.target then
        self:updateTargetAuras(true)
    end
end

function DHUD2:UNIT_TARGET(who)
    self:Debug("UNIT_TARGET", who)
    local p = DHUD2.db.profile
    if who == "target" then
        if UnitExists("targetTarget") and not UnitIsUnit("targettarget", "player") then
            local color = {r=0,g=0,b=0,hex=""}
            local maxVal, curVal, curFrac
            if currLayout.targetTargetHealth ~= "e" then
                maxVal, curVal, curFrac = getHealth("targettarget")
                color.r, color.g, color.b, color.hex = self:colorize("targetTarget", "health",  curFrac)
                self:updateBar(currLayout.targetTargetHealth, maxVal, curVal, curFrac, color)            
            end
            if currLayout.targetTargetPower ~= "e" then
                maxVal, curVal, curFrac = getPower("targettarget")
                color.r, color.g, color.b, color.hex = self:colorize("targetTarget", UnitPowerType("targetarget"),  curFrac)
                self:updateBar(currLayout.targetTargetPower, maxVal, curVal, curFrac, color)            
            end
            self:updateText("targetTarget", p.show.targetTargetName)
        else
            if currLayout.targetTargetHealth ~= "e" then
                self:updateBar(currLayout.targetTargetHealth, false)            
            end
            if currLayout.targetTargetPower ~= "e" then
                self:updateBar(currLayout.targetTargetPower, false)            
            end
            DHUD2_targetTarget_Text:Hide()
        end
    end
    self:updateAlpha()
end

function DHUD2:UNIT_HAPPINESS(who)
    if who == "pet" then
        local happ = GetPetHappiness()
        local health = UnitHealth(who)
        self:updatePetIcon(health, happ)
    end
end

function DHUD2:UNIT_PET(who)
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile
    if who == "player" then
        hasPet = UnitExists("pet")
        DHUD2_lpi:Hide()
        DHUD2_lpo:Hide()
        DHUD2_rpi:Hide()
        DHUD2_rpo:Hide()
        if hasPet and p.show.pet then
            maxVal, curVal, curFrac, needH = getHealth("pet")
            color.r, color.g, color.b, color.hex = self:colorize("pet", "Health", curVal)
            self:updateBar(currLayout.petHealth, maxVal, curVal, curFrac)
            maxVal, curVal, curFrac, needP = getPower("pet")
            local typ = UnitPowerType(who)
            color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curVal)
            self:updateBar(currLayout.petPower, maxVal, curVal, curFrac)
            local happ = GetPetHappiness()
            self:updatePetIcon(curVal, happ)
        else
            self:updatePetIcon(0, false)
            self:updateBar(currLayout.petHealth, false)
            self:updateBar(currLayout.petPower, false)
        end
    end
    self:updateAlpha()
end

function DHUD2:RAID_TARGET_UPDATE()
    self:updateTargetIcon()
end

function DHUD2:PARTY_LOOT_METHOD_CHANGED()
    local _, master = GetLootMethod()
    if master == 0 then
        DHUD2_playerLooter:Show()
    else
        DHUD2_playerLooter:Hide()
    end
end

function DHUD2:PARTY_MEMBERS_CHANGED()
    if GetNumPartyMembers() > 0 then
        if IsPartyLeader() then
            DHUD2_playerLeader:Show()
        else
            DHUD2_playerLeader:Hide()
        end
    else
        DHUD2_playerLeader:Hide()
        DHUD2_playerLooter:Hide()
    end
    
end

function DHUD2:PLAYER_FLAGS_CHANGED()
    --- SERA?
end

function DHUD2:UNIT_SPELLCAST_START(who)
    self:Debug("UNIT_SPELLCAST_START", who)
    if who == "player" then        
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:initCast(currLayout.playerCastBar, startTime, endTime, true)
        playerCast = true
    elseif who == "target" then        
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:initCast(currLayout.targetCastBar, startTime, endTime, true)
    end
end

function DHUD2:UNIT_SPELLCAST_STOP(who)
    self:Debug("UNIT_SPELLCAST_STOP", who)
    if who == "player" and playerCast then
        playerCast = self:stopCast(currLayout.playerCastBar, true)
    elseif who == "target" then
        self:stopCast(currLayout.targetCastBar, true)
    end
end

function DHUD2:UNIT_SPELLCAST_DELAYED(who)
    self:Debug("UNIT_SPELLCAST_DELAYED")
    if who == "player" then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.playerCastBar, startTime, endTime)
    elseif who == "target" then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.targetCastBar, startTime, endTime)
    end
    
end

function DHUD2:OnSpellFail(who)
    self:Debug("OnSpellFail", who)
    if who == "player" and playerCast then
        playerCast = self:stopCast(currLayout.playerCastBar, true)
    elseif who == "target" then
        self:stopCast(currLayout.targetCastBar, true)  
    end
end

function DHUD2:UNIT_SPELLCAST_CHANNEL_START(who)
    self:Debug("UNIT_SPELLCAST_CHANNEL_START")
    if who == "player" then
        local _, _, _, _, startTime, endTime = UnitChannelInfo(who)
        self:initCast(currLayout.playerCastBar, startTime, endTime)
        playerCast = true
    elseif who == "target" then
        local _, _, _, _, startTime, endTime = UnitChannelInfo(who)
        self:initCast(currLayout.targetCastBar, startTime, endTime)
    end
end

function DHUD2:UNIT_SPELLCAST_CHANNEL_STOP(who)
    self:Debug("UNIT_SPELLCAST_CHANNEL_STOP")
    if who == "player" then
        playerCast = self:stopCast(currLayout.playerCastBar)
    elseif who == "target" then
        self:stopCast(currLayout.targetCastBar)
    end
end


function DHUD2:UNIT_SPELLCAST_CHANNEL_UPDATE(who)
    self:Debug("UNIT_SPELLCAST_CHANNEL_UPDATE")
    if who == "player" and playerCast then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.playerCastBar, startTime, endTime)
    elseif who == "target" then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.targetCastBar, startTime, endTime)
    end
end

function DHUD2:UNIT_DISPLAYPOWER(who)
    self:Debug("UNIT_DISPLAYPOWER", who)
    local p = DHUD2.db.profile
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    if who == "player" then
        if hasPet then 
            return
        elseif isDruid and SmartyCat then
            if UnitPowerType("player") == 0 then
                self:updateBar(currLayout.druidMana, false)
                self:UnregisterEvent("SmartyCat_ManaUpdated")
            else
                _G["DHUD2_"..currLayout.druidMana]:Show()
                self:RegisterEvent("SmartyCat_ManaUpdated", "OnFormUpdateMana")
                curVal, maxVal = SmartyCat.GetCurrMana(), SmartyCat.GetMaxMana()
                color.r, color.g, color.b, color.hex = self:colorize("player", 0, curVal/maxVal)
                self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color)
            end
        end
        maxVal, curVal, curFrac, need = getPower("player")
        color.r, color.g, color.b, color.hex = self:colorize("player", UnitPowerType("player"), curFrac)
        self:updateBar(currLayout.playerPower, maxVal, curVal, curFrac, color)
        self:updateAlpha()
    end
end

function DHUD2:OnFormUpdateMana(curVal, maxVal)
    if hasPet then
        return
    end
    local color = {r=0,g=0,b=0,hex=""}
    color.r, color.g, color.b, color.hex = self:colorize("player", 0, curVal/maxVal)
    self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color)
    self:updateAlpha()
end

function DHUD2:OnTooltipUpdate()
   Tablet:SetHint(L["Open DHUD2's configuration window"])
end
