--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]

-- Texts
local _G = _G
local string_sub = string.sub
local string_gsub = string.gsub
local string_len = string.len
local string_format = string.format
local math_floor = math.floor
local styles = {
    values = {
        ["Full"] = "<value>/<maxval><percent>",
        ["Simple"] = "<value>/<maxval>",
        ["Small"] = "<value><percent>",
        ["Percent"] = "<percent>",
        ["Minimal"] = "<value>",
        },
    names = {
        ["Full"] = "<level><elite> <name> [ <class><pet><npc> ]",
        ["Simple"] = "<level><elite> <name>",
        ["Small"] = "<name> [ <class><pet><npc> ]",
        ["Minimal"] = "<level> <name>",
    },
}
local classColors = {
    ["HUNTER"] = "ABD473", 
    ["WARLOCK"] = "9482C9",
    ["PRIEST"] = "FFFFFF",
    ["PALADIN"] = "F58CBA",
    ["MAGE"] = "69CCF0",
    ["ROGUE"] = "FFF569",
    ["DRUID"] = "FF7D0A",
    ["SHAMAN"] = "00DBBA",
    ["WARRIOR"] = "C79C6E",
}
local defaultfont = "Fonts/FRIZQT__.TTF"
local defaultfont_num = "Interface\\AddOns\\DHUD\\textures\\Number.TTF"


local getReactionColor
do
    local reacColors = {
        "ff0000",
        "ffff00",
        "55ff55",
        "6d6dff",
        "008800",
        "cccccc",
        }
    function getReactionColor(unit)
    	local color
        if (UnitIsTapped(unit) and (not UnitIsTappedByPlayer(unit))) then
    		color = 6
    	elseif (UnitIsPlayer(unit)) then
    		if (UnitIsPVP(unit)) then
                color = UnitCanAttack("player", unit) and 1 or 5
    		else
    			if (UnitCanAttack("player", unit) or UnitFactionGroup(unit) ~= UnitFactionGroup("player")) then
    				color = 2
    			else
    				color = 4
    			end
    		end
    	else
    		local reaction = UnitReaction(unit, "player");
    		if (reaction) then
    			if (reaction < 4) then
    				color = 1
    			elseif (reaction == 4) then
    				color = 2
    			else
    				color = 3
    			end
    		end
    	end
    	return reacColors[color];
    end
    
end

function DHUD2:updateBarText(bar, maxval, curval, frac, color)
    local p = DHUD2.db.profile
    local percent = math_floor(frac * 100)
    local font = _G["DHUD2_"..bar.."_Text_Text"]
    local text = styles.values[p.styles[bar]]
    text = string_gsub(text, "<value>", "|cff"..color..curval)
    if text:find("|c") then
        text = string_gsub(text, "<maxval>", maxval)
    else
        text = string_gsub(text, "<maxval>", "|cff"..color..maxval)
    end
    if text:find("|c") then
        text = string_gsub(text, "<percent>", "|r |cff747070(|r"..percent.."|cff747070)|r%%")
    else
        text = string_gsub(text, "<percent>", "|cff747070(|r"..percent.."|cff747070)|r%%")
    end
    font:SetText(text)
    font:SetWidth(2000)
    font:SetJustifyH("RIGHT")
    local w = font:GetStringWidth() + 10
    font:SetWidth(w)
    _G["DHUD2_"..bar.."_Text"]:SetWidth(w)
    _G["DHUD2_"..bar.."_Text"]:Show()
end

function DHUD2:hideBarText(bar)
    _G["DHUD2_"..bar.."_Text"]:Hide()
end



--Upodate Target and TargetTarget text
function DHUD2:updateText(unit, show)
    if show then
        local p = DHUD2.db.profile
        local font = _G["DHUD2_"..unit.."_Text_Text"]
        local text = styles.names[p.styles[unit]]
        local level = UnitLevel(unit)
        local levelText, addLevel
        levelText = level < 0 and "??" or level
        level = level < 0 and 99 or level
        local levColor = GetDifficultyColor(level)
        local color = self:rgb2hex(levColor.r, levColor.g, levColor.b)
        text, addLevel = string_gsub(text, "<level>", "|cff"..color..levelText)
        local _, eliteText = self:checkElite(unit)
        if addLevel == 1 then
            text = string_gsub(text, "<elite>", ""..eliteText.."|r")
        else
            text = string_gsub(text, "<elite>", "|cff"..color..eliteText.."|r")
        end
        text = string_gsub(text, "<name>", "|cff"..getReactionColor(unit)..UnitName(unit).."|r")
        local localClass, class = UnitClass(unit)
        color = classColors[class]        
        if localClass and ( not(self:unitIsNPC(unit) or self:unitIsPet(unit)) ) then            
            if not UnitIsPlayer(unit) then
                localClass = UnitCreatureType(unit)
            end
            text = string_gsub(text, "<class>", "|cff"..color..localClass.."|r")
        else
            text = string_gsub(text, "<class>", "")
        end
        if self:unitIsPet(unit) then
            text = string_gsub(text, "<pet>", "|cff"..color.."Pet".."|r")
        else
            text = string_gsub(text, "<pet>", "")
        end
        if self:unitIsNPC(unit) then
            color = "ffff9a"
            text = string_gsub(text, "<npc>", "|cff"..color.."NPC".."|r")            
        else
            text = string_gsub(text, "<npc>", "")
        end
        font:SetText(text)
        font:SetWidth(4000)
        local w = font:GetStringWidth() + 10
        font:SetWidth(w)
        _G["DHUD2_"..unit.."_Text"]:SetWidth(w)
        _G["DHUD2_"..unit.."_Text"]:Show()
    else
        _G["DHUD2_"..unit.."_Text"]:Hide()
    end
end

function DHUD2:updateCastText(bar, value)
    --DHUD2:Debug("updateCastText")
    if value > 0 then 
        local font = _G["DHUD2_"..bar.."_Text_Text"]
        local castText = string_format("%.1f", value)
        font:SetText(castText)
        font:SetWidth(2000)
        local w = font:GetStringWidth() + 10
        font:SetWidth(w)
        _G["DHUD2_"..bar.."_Text"]:SetWidth(w)
        _G["DHUD2_"..bar.."_Text"]:Show()
    else
        _G["DHUD2_"..bar.."_Text"]:Hide()
    end
end

function DHUD2:updateDelayText(bar, value)
    if value > 0 then
        local font = _G["DHUD2_"..bar.."d_Text_Text"]
        local delayText = string_format("%.1f", value)
        font:SetTextColor(1,0,0)
        font:SetText(delayText)
        font:SetWidth(2000)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        _G["DHUD2_"..bar.."d_Text"]:SetWidth(w)
        _G["DHUD2_"..bar.."d_Text"]:Show()
    else        
        _G["DHUD2_"..bar.."d_Text"]:Hide()
    end
end

function DHUD2:setFonts()
        local p = self.db.profile
        local font
        local w
        for i = 1, 16 do
            font = _G["DHUD2_playerBuff"..i.."_Text"]
            font:SetFont(defaultfont_num, p.fontsize.playerBuff * p.gral.scale, p.outlines.playerBuff)
            w = font:GetStringWidth() + 10
            font:SetWidth(w)
        end
        --Targer Text
        DHUD2_target_Text_Text:SetFont(defaultfont, p.fontsize.targetName * p.gral.scale, p.outlines.target)
        w = DHUD2_target_Text_Text:GetStringWidth() + 10
        DHUD2_target_Text_Text:SetWidth(w)
        --Targer Target Text
        DHUD2_targetTarget_Text_Text:SetFont(defaultfont, p.fontsize.targetTargetName * p.gral.scale, p.outlines.targetTarget)
        w = DHUD2_targetTarget_Text_Text:GetStringWidth() + 10
        DHUD2_targetTarget_Text_Text:SetWidth(w)
        -- Cast and delay Text
        DHUD2_rc_Text_Text:SetFont(defaultfont, p.fontsize.castTime * p.gral.scale, p.outlines.castTime)
        w = DHUD2_rc_Text_Text:GetStringWidth() + 10
        DHUD2_rc_Text_Text:SetWidth(w)
        
        DHUD2_lc_Text_Text:SetFont(defaultfont, p.fontsize.castTime * p.gral.scale, p.outlines.castTime)
        w = DHUD2_lc_Text_Text:GetStringWidth() + 10
        DHUD2_lc_Text_Text:SetWidth(w)
        
        DHUD2_rcd_Text_Text:SetFont(defaultfont, p.fontsize.castDelay * p.gral.scale, p.outlines.castDelay)
        w = DHUD2_rcd_Text_Text:GetStringWidth() + 10
        DHUD2_rcd_Text_Text:SetWidth(w)
        
        DHUD2_lcd_Text_Text:SetFont(defaultfont, p.fontsize.castDelay * p.gral.scale, p.outlines.castDelay)
        w = DHUD2_lcd_Text_Text:GetStringWidth() + 10
        DHUD2_lcd_Text_Text:SetWidth(w)
        
        -- Bar's Text
        local l = self.currLayout
        font = _G["DHUD2_"..l.playerHealth.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.player * p.gral.scale, p.outlines.player)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        font = _G["DHUD2_"..l.playerPower.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.player * p.gral.scale, p.outlines.player)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        font = _G["DHUD2_"..l.targetHealth.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.target * p.gral.scale, p.outlines.target)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        font = _G["DHUD2_"..l.targetPower.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.target * p.gral.scale, p.outlines.target)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        font = _G["DHUD2_"..l.petHealth.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.pet * p.gral.scale, p.outlines.pet)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        font = _G["DHUD2_"..l.petPower.."_Text_Text"]
        font:SetFont(defaultfont_num, p.fontsize.pet * p.gral.scale, p.outlines.pet)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        
        if l.targetTargetHealth ~= "e" then
            font = _G["DHUD2_"..l.targetTargetHealth.."_Text_Text"]
            font:SetFont(defaultfont_num, p.fontsize.targetTarget * p.gral.scale, p.outlines.targetTarget)
            w = font:GetStringWidth() + 10
            font:SetWidth(w)
        end
        
        if l.targetTargetPower ~= "e" then
            font = _G["DHUD2_"..l.targetTargetPower.."_Text_Text"]
            font:SetFont(defaultfont_num, p.fontsize.targetTarget * p.gral.scale, p.outlines.targetTarget)
            w = font:GetStringWidth() + 10
            font:SetWidth(w)
        end
        
        if l.focusHealth ~= "e" then
            font = _G["DHUD2_"..l.focusHealth.."_Text_Text"]
            font:SetFont(defaultfont_num, p.fontsize.focus * p.gral.scale, p.outlines.focus)
            w = font:GetStringWidth() + 10
            font:SetWidth(w)    
        end
        
        if l.focusPower ~= "e" then
            font = _G["DHUD2_"..l.focusPower.."_Text_Text"]
            font:SetFont(defaultfont_num, p.fontsize.focus * p.gral.scale, p.outlines.focus)
            w = font:GetStringWidth() + 10
            font:SetWidth(w)
        end
end
